#include "ram_caching.hpp"

namespace RtcMemory{
    // 2D array of 20 Strings with maxLength of 251
    RTC_DATA_ATTR char storage[maxLength_][maxSize_];
    RTC_DATA_ATTR int storedElements;
    RTC_DATA_ATTR int headElement = 0;
    RTC_DATA_ATTR int tailElement = 0;

    static const char* TAG = "CACHING";


    void store(std::string message){

        // turn data into char array
        const char* jsonString = message.c_str();
    
        // move head to new element
        headElement = (headElement + 1) % maxSize_;
        // apparently I am not allowed to copy to rtc mem
        for(int i=0; i<maxLength_; i++){
            storage[i][headElement] = jsonString[i];
            if(jsonString[i] == '\0'){
                break;
            }
        }
        storedElements++;
        ESP_LOGE(TAG, "Moved message to storage.");
    }

    String get_from_storage(){
        // remove element pointed at by tail
        String buf = "";
        char current = '\0';
        for(int i = 0; i<maxLength_; i++){
            current = storage[i][tailElement];
            buf += current;
            if(current == '\0'){
                break;
            }
        }
        // move tail to next element
        tailElement = (tailElement + 1) % maxSize_;
        storedElements--;
        ESP_LOGE(TAG, "Retrieved message from storage");
        return buf;
    }

    bool is_full(){
        return headElement == tailElement;
    }

    int stored_amount(){
        return storedElements;
    }
}
