#include "ESPNow.hpp"

uint8_t BROADCAST_MAC[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
esp_now_peer_info_t hostInfo;
Preferences preferences;

void get_host_mac(uint8_t *destination)
{
	preferences.begin("config", true);
	if (!preferences.isKey("host")) {
		preferences.getBytes("host", destination, sizeof(uint8_t) * 6);
	} else {
		memcpy(destination, BROADCAST_MAC, sizeof(BROADCAST_MAC));
		Serial.println("backup mac used");
	}
	preferences.end();
}
void on_data_sent(const uint8_t *mac_addr, esp_now_send_status_t status)
{
	// go to sleep
}

void on_data_recv(const uint8_t *mac, const uint8_t *incomingData, int len)
{
	Serial.println("message recieved");
	config new_config;
	memcpy(&new_config, incomingData, sizeof(new_config)); // TODO: check for valid mac

	// put the host address in flash mem
	preferences.begin("config", false);
	if (!preferences.isKey("host")) {
		preferences.putBytes("host", new_config.host, sizeof(new_config.host));
		Serial.println("host mac saved to flash");
	} else{
		Serial.println("host mac already exists");
	}// host change shouldn't be an issue
	preferences.end();
	// sync time
	Time::getInstance().setTime(
	    new_config.time_millis); // see https://www.esp32.com/viewtopic.php?t=9965, maybe this needs an offset
	Serial.println("Saved Time: " + (String) new_config.time_millis);
	Serial.flush();
}

esp_err_t espnow_setup()
{
	esp_err_t result;
	WiFi.mode(WIFI_STA);
	result = esp_now_init();
	if (result != ESP_OK) {
		// initialization failed
		return result; // not sure about this
	}

	get_host_mac(hostInfo.peer_addr); // check if there is a host saved in flash mem, broadcast otherwise

	hostInfo.channel = 0;
	hostInfo.encrypt = 0;
	esp_now_add_peer(&hostInfo);

	esp_now_register_recv_cb(on_data_recv);
	esp_now_register_send_cb(on_data_sent);

	return ESP_OK;
}

esp_err_t espnow_send_message(const Message& message){
	Serial.println("sending Message");
	esp_err_t success;
	ClientDataPackage dataP = message.get_client_data_package();
	uint8_t recipient;
	get_host_mac(&recipient);

	success = esp_now_send(&recipient, (uint8_t *) &dataP, sizeof(ClientDataPackage));
	// if(success != ESP_OK){
	//     if(!ram_cache_is_full()){
	//         ram_cache_push(*data);
	//     }
	// }

	for (int i = 0; i < dataP.amountData; i++) {
		Serial.println(dataP.values[i]);
	}
	
	Serial.println((String) "time sent: " + dataP.timestamp);
	Serial.println((String) "Send status: " + success);
	Serial.println();
	Serial.println("done");
	Serial.flush();
	return success;
}
