#include <esp_now.h>
#include <Preferences.h>
#include <ESP32Time.h>
#include "WiFi.h"
#include "espnow.hpp"
#include "ram_caching.hpp"

uint8_t BROADCAST_MAC[] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
esp_now_peer_info_t hostInfo;
Preferences preferences;
ESP32Time rtc;
void get_host_mac(uint8_t* destination){
    preferences.begin("config", true);
    if(preferences.isKey("host")){
        preferences.getBytes("host", destination, sizeof(uint8_t) * 6);
    }
    else{
        memcpy(destination, BROADCAST_MAC, sizeof(BROADCAST_MAC));
        Serial.println("backup mac used");
    }
    preferences.end();
}
void on_data_sent(const uint8_t *mac_addr, esp_now_send_status_t status){
    // go to sleep
}

void on_data_recv(const uint8_t *mac, const uint8_t *incomingData, int len){
    Serial.println("message recieved");
    config new_config;
    memcpy(&new_config, incomingData, sizeof(new_config)); // TODO: check for valid mac

    // put the host address in flash mem
    preferences.begin("config", false);
    if(!preferences.isKey("host")){
        preferences.putBytes("host", new_config.host, sizeof(new_config.host));
        Serial.println("host mac saved to flash");
    }// host change shouldn't be an issue
    preferences.end();
    // sync time
    rtc.setTime(new_config.time_millis); // see https://www.esp32.com/viewtopic.php?t=9965, maybe this needs an offset
}



esp_err_t espnow_setup(){
    esp_err_t result;
    WiFi.mode(WIFI_STA);
    result = esp_now_init();
    if(result != ESP_OK){
        //initialization failed
        return result; // not sure about this
    }
    
    get_host_mac(hostInfo.peer_addr); // check if there is a host saved in flash mem, broadcast otherwise

    hostInfo.channel = 0;
    hostInfo.encrypt = 0;
    esp_now_add_peer(&hostInfo);

    esp_now_register_recv_cb(on_data_recv);
    esp_now_register_send_cb(on_data_sent);

    return ESP_OK;
}

void Message::add_data(float value, int identifier){
    if(data.amountData < NUM_SENSORS){
        data.values[data.amountData] = value;
        data.identifiers[data.amountData] = identifier;
        data.amountData++;
    }
}

esp_err_t Message::send(){
    Serial.println("sending Message");
    esp_err_t success;
    success = esp_now_send(recipient,  (uint8_t * ) &data , sizeof(data));
    // if(success != ESP_OK){
    //     if(!ram_cache_is_full()){
    //         ram_cache_push(*data);
    //     }
    // }
    for(int i=0; i<data.amountData; i++){
        Serial.print("identifier: ");
        Serial.print(data.identifiers[i]);
        Serial.print("  value: ");
        Serial.println(data.values[i]);
    }
    Serial.println((String) "time sent: " + data.timestamp);
    Serial.println((String) "Send status: " + success);
    Serial.println();
    Serial.flush();
    Serial.println("done");
    return success;
}

Message :: Message(){
    // data = (data_struct*) malloc(sizeof(data_struct));
    
    // check for existing host mac address, use broadcast otherwise
    get_host_mac(recipient);

    data.amountData = 0;
    data.timestamp = rtc.getMillis(); // I am assuming we are not sending data from Unix Epoch
}

Message :: Message(data_struct old_data){
    memcpy(&data, &old_data, sizeof(data));
    get_host_mac(recipient);
}

// Message :: ~Message(){
//     free((void*) data);
// }

