#include "FS.h"
#include "SD.h"
#include "SPI.h"
#include <Arduino.h>
#include <WiFi.h>
#include <esp_now.h>
#include <sys/unistd.h>

#define RXD2 18
#define TXD2 19

long time1 = 0;

uint8_t hostMac[] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};

//SoftwareSerial mySerial(RXD2, TXD2);

enum MessageType{
    dataAck,
    hostChange
};
typedef struct response{
	MessageType type;
	uint8_t mac[6];
	long time;
}response;

void print_mac(const uint8_t *mac){
		for (int i = 0; i < 6; i++) {
		Serial.print(mac[i], HEX);
		Serial.print(":");
	}
	Serial.println();
	Serial.flush();
}

void on_data_sent(const uint8_t *mac_addr, esp_now_send_status_t status)
{
	// go to sleep
}

void on_data_recv(const uint8_t *mac, const uint8_t *incomingData, int len)
{
	// print mac
	Serial.print("Message recieved from: ");
	print_mac(mac);

	char data[len];
	memcpy(data, incomingData, len);
	Serial.println(data);
	Serial1.write(data);

	// add client if not already peer
	if(!esp_now_is_peer_exist(mac)){ 
		Serial.println("adding client to peers");

		esp_now_peer_info_t client = {};
		memcpy(client.peer_addr, mac, 6);
		client.encrypt = false;
		client.channel = 0;

		esp_err_t status = esp_now_add_peer(&client);
		if(status != ESP_OK){
			Serial.print("Failed to add peer: ");
			Serial.println(status, HEX);
		}
		Serial.flush();
	}

	// respond with own mac + time
	response response;
	response.type = dataAck;
	esp_read_mac(response.mac, ESP_MAC_WIFI_STA);
	response.time = 7 * time1++;
	esp_err_t success = esp_now_send(mac, (uint8_t *) &response, sizeof(response));
	if(success != ESP_OK){
		Serial.print("Failed to respond: ");
		Serial.println(success, HEX);
	}
	else{
		Serial.println("Response sent: " + (String) response.time);
	}
	Serial.println("\n");
}

void setup()
{
	Serial.begin(115200);
	Serial1.begin(115200, SERIAL_8N1, RXD2, TXD2);

	WiFi.mode(WIFI_STA);
	Serial.println("ESPNow init");
	if (esp_now_init() != ESP_OK) {
		// initialization failed
		Serial.println("ESPNow init failed");
		return; // not sure about this
	}
	Serial.println("ESPNow init success");

	// tell all clients that this is now the host
	// can probably be removed after testing
	response hello;
	hello.type = hostChange;
	esp_read_mac(hello.mac, ESP_MAC_WIFI_STA);
	hello.time = 0;
	esp_now_send(hostMac, (uint8_t*) &hello, sizeof(hello));

	esp_now_register_recv_cb(on_data_recv);
	//  write your initialization code here
}

void loop() {
	Serial.print("This device: ");
	Serial.println(WiFi.macAddress());
	Serial.println("\n");
	delay(1000000);
}